% master file for empirical application

% This code was developed for the following paper: Mayer, A., and D. Wied,
% 2021: "Estimation and Inference in Factor Copula Models with Exogenous Covariates"

% Several parts of the code use ...
% 1) Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.
% 2) Kevin Sheppards MFE toolbox
% (https://www.kevinsheppard.com/code/matlab/mfe-toolbox/)

rawdata = importdata('~marginals/residuals_all4.csv'); %import fitted residuals
 
estclass = "benchmark"; 
esttype  = "typeA1";
eps_dist = 'st';
model = [estclass, esttype, eps_dist];

q = [5 10 90 95]/100;

if strcmp(esttype, 'typeA1')    
    LB           =   [.01 -.99 0.*ones(1,1)'];
    UB           =   [.49  .99 5.*ones(1,1)'];
elseif strcmp(esttype, "typeA2") 
    LB           =   [.01 -.99 0.*ones(4,1)'];
    UB           =   [.49  .99 5.*ones(4,1)'];
elseif strcmp(esttype, "typeA3") || strcmp(esttype, "typeB1") || strcmp(esttype, "typeB2")
    LB           =   [.01 -.99 0.*ones(5,1)'];
    UB           =   [.49  .99 5.*ones(5,1)'];
elseif strcmp(esttype, "typeA4")
    LB           =   [.01 -.99 0.*ones(8,1)'];
    UB           =   [.49  .99 5.*ones(8,1)'];
elseif strcmp(esttype, "typeB3") || strcmp(esttype, "typeB4")
    LB           =   [.01 0.*ones(8,1)'];
    UB           =   [.49 5.*ones(8,1)'];
end

% blocks
selBlks1 = 1:13; 
selBlks2 = 14:24;
selBlks3 = 25:35;
selBlks4 = 36:43;
sel =  [selBlks1, selBlks2, selBlks3, selBlks4]; 

if strcmp(esttype, 'typeA1')
    blks = ones(1,length(sel));
else
    blks = [ones(1,length(selBlks1)),2*ones(1,length(selBlks2)),3*ones(1,length(selBlks3)),4*ones(1,length(selBlks4))]; %
end

data_resid = rawdata(:,sel);
[T,N]=size(data_resid);
data_uresid_semipar = tiedrank(data_resid)/(T+1);       
Z_hat = log(abs(rawdata(:,end))); % "gold" factor

S  = 25; S1 = 10;
B  = 2000*T;    
rhoS =  1;
rng_type = 'threefry4x64_20';% any 'threefry4x64_20';%'twister';%'mlfg6331_64';% 


Weight = 1;
seed = 1234;
other_est  = [T S N];

reg  = Z_hat*strcmp(estclass, 'estimable')+0;%; %
exog = 1*strcmp(estclass, 'estimable');


fun = @(x) objfunc(x, other_est,  data_uresid_semipar, Weight, q,...
                                     reg, seed, blks, rng_type,rhoS,model, 'false'); 


fun0 = @(x) initials(x,data_uresid_semipar,Weight,q, reg,...
                     blks,LB,UB,T,N,S1,rhoS,rng_type,seed, model);
opts = optimoptions('surrogateopt','UseParallel',false,'PlotFcn',[],'display','off'); 

[theta_ini_1,Q_ini_1,~,~] = surrogateopt(fun,LB,UB, opts);
[theta_ini_2,Q_ini_2] = fun0(theta_ini_1); 

if Q_ini_2 < Q_ini_1
    theta_ini = theta_ini_2;
else
    theta_ini = theta_ini_1;
end

% ps_opts =  optimoptions('patternsearch','display','off', 'UseCompletePoll', true, 'UseVectorized', false, 'UseParallel', true);
ps_opts =  optimoptions('patternsearch','display','off');

[theta_ini_3b,Q_ini_3b,~,~] = fminsearchbnd(fun, theta_ini,LB,UB);
[theta_ini_3a,Q_ini_3a,~,~] = patternsearch(fun, theta_ini_3b,[],[],[],[],LB,UB,[],ps_opts);

format long

if Q_ini_3b < Q_ini_3a
    theta_ini_3 = theta_ini_3b;
else
    theta_ini_3 = theta_ini_3a;
end
theta_hat = theta_ini_3;
 
format short

[~,Xsim]  = estimation(theta_hat, other_est, seed, reg, rng_type, blks, model, 'false');
Sigma_hat = var_est_blk(data_resid,Xsim,q,B,seed,blks,exog,var_type,rhoS);  
clear Xsim
      
[J_hat05,~] = num_deriv_blk(theta_hat,data_uresid_semipar,other_est,Weight,blks,q, reg,seed,0.05,LB,UB,rng_type,rhoS,model);
[J_hat01,~] = num_deriv_blk(theta_hat,data_uresid_semipar,other_est,Weight,blks,q, reg,seed,0.01,LB,UB,rng_type,rhoS,model);

Omega_hat_05   = (J_hat05'*Weight*J_hat05)\(J_hat05'*Weight*Sigma_hat*Weight*J_hat05)/(J_hat05'*Weight*J_hat05);
Omega_hat_01   = (J_hat01'*Weight*J_hat01)\(J_hat01'*Weight*Sigma_hat*Weight*J_hat01)/(J_hat01'*Weight*J_hat01);
[~,ghat] = objfunc(theta_hat,other_est,data_uresid_semipar,Weight,q,reg,seed,blks,rng_type,rhoS,model, 'false');

[J05, p05,~,Jstat05] = JTest(ghat,T,2000,Weight,Sigma_hat,J_hat05,seed);
[J01, p01,~,Jstat01] = JTest(ghat,T,2000,Weight,Sigma_hat,J_hat01,seed);

test05 = sqrt(T)*theta_hat./sqrt(diag(Omega_hat_05))';
test01 = sqrt(T)*theta_hat./sqrt(diag(Omega_hat_01))';
 

Weight_opt = inv(Sigma_hat);
fun_opt = @(x) objfunc(x, other_est,  data_uresid_semipar, Weight_opt, q,reg, seed, blks, rng_type,rhoS,model, 'false');

[theta_hat_opt2,Q_opt2,~,~] = fminsearchbnd(fun_opt, theta_hat,LB,UB);
[theta_hat_opt1,Q_opt1,~,~] = patternsearch(fun_opt, theta_hat_opt2,[],[],[],[],LB,UB,[],ps_opts);


if Q_opt1 < Q_opt2
    theta_hat_opt = theta_hat_opt1;
else
    theta_hat_opt = theta_hat_opt2;
end

[J_hat05,~] = num_deriv_blk(theta_hat_opt,data_uresid_semipar,other_est,Weight_opt,blks,q, reg,seed,0.05,LB,UB,rng_type,rhoS,model);
[J_hat01,~] = num_deriv_blk(theta_hat_opt,data_uresid_semipar,other_est,Weight_opt,blks,q, reg,seed,0.03,LB,UB,rng_type,rhoS,model);

Omega_hat_opt_05   = inv(J_hat05'*Weight_opt*J_hat05);
Omega_hat_opt_01   = inv(J_hat01'*Weight_opt*J_hat01);
[~,ghat] = objfunc(theta_hat_opt,other_est,data_uresid_semipar,Weight_opt,q,reg,seed,blks,rng_type,rhoS,model, 'false');

[J_opt_05, p_opt_05,~,J_opt_stat05] = JTest(ghat,T,2000,Weight_opt,Sigma_hat,J_hat05,seed);
[J_opt_01, p_opt_01,~,J_opt_stat01] = JTest(ghat,T,2000,Weight_opt,Sigma_hat,J_hat01,seed);

test_opt_05 = sqrt(T)*theta_hat_opt./sqrt(diag(Omega_hat_opt_05))';
test_opt_01 = sqrt(T)*theta_hat_opt./sqrt(diag(Omega_hat_opt_01))';

[J05 J01]
[Jstat05 Jstat01]
[p05 p01]

theta_hat
test05
test01

[J_opt_05 J_opt_01]
[J_opt_stat05 J_opt_stat01]
[p_opt_05 p_opt_01]

theta_hat_opt
test_opt_05
test_opt_01
 
% 1-chi2cdf(J_opt_stat05,...)