% file to filter stock returns

% This code was developed for the following paper: Mayer, A., and D. Wied,
% 2021: "Estimation and Inference in Factor Copula Models with Exogenous Covariates"

% Several parts of the code use ...
% Kevin Sheppards MFE toolbox
% (https://www.kevinsheppard.com/code/matlab/mfe-toolbox/)

rawdata = importdata('~/returns_all4.csv');
 
x = rawdata.data(:,end); % 43 stocks
y = rawdata.data(:,1:(end-1));% gold factor
[T, N] = size(y);



q = [.1 .25 .5 .75 .9];
quantile(mean(y),q)
quantile(std(y),q) 
quantile(skewness(y),q)
quantile(kurtosis(y),q)

% check wheteher Z has sign. conditional mean (AR(1)) dynamics (no)
estmdl = fitlm(x(1:(end-1)),x(2:end));
esthac = hac(estmdl,'type','HC','weights','HC0','display','off');

r = estmdl.Coefficients.Estimate;
R = [1 0;0 1];
[W,p] = waldtest(r,R,esthac);
%

%%%%%
data_store=NaN(T-1,N);
coef_store=NaN(9,N);
tstat_store=NaN(9,N);

for ii=1:N
    yy = y(:,ii);
    [~,para_temp,~,ster1,resid_temp,~]=fitARMAGARCH0(yy,x,[1,0,1,1,1],'SKEWT',[]); 
    coef_store(:,ii) = para_temp;
    data_store(:,ii) = resid_temp;
    tstat_store(:,ii) = para_temp./ster1;
end
%%%%
% outpout
% const, AR(1), xreg
quantile(coef_store(1,:),q) 
quantile(coef_store(2,:),q) 
quantile(coef_store(3,:),q) 
% omega, alpha, gamma, beta 
quantile(coef_store(4,:),q)%const
quantile(coef_store(5,:),q)%ARCH
quantile(coef_store(6,:),q)%leverage
quantile(coef_store(7,:),q)%garch 
% nu, lamba
quantile(coef_store(8,:),q)%nu
quantile(coef_store(9,:),q)%lambda
xx = x(2:T);
[par,~,HT,VCVROBUST,~,~,~]= tarch(xx,1,1,1,'SKEWT',2,[],[]);
x_hat=xx./sqrt(HT);

mean(xx)
std(xx)
skewness(xx)
kurtosis(xx) 


data = [data_store(2:end,:) x_hat(1:(end-1))];
writematrix(data(1:1460,:),'~/residuals_all4.csv');




