% master file for MC study

% This code was developed for the following paper: Mayer, A., and D. Wied,
% 2021: "Estimation and Inference in Factor Copula Models with Exogenous Covariates"

% Several parts of the code use ...
% 1) Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.
% 2) Kevin Sheppads MFE toolbox
% (https://www.kevinsheppard.com/code/matlab/mfe-toolbox/)

M = 5; % number of MC replications

q            =   [5 10 90 95]/100;% quantiles for quantile dependence measure
theta0       =   [.25   -.5 1/2 1 1.5 2]; % true value
LB           =   [.01   -.99 0.*ones(4,1)'];
UB           =   [.49    .99 5.*ones(4,1)'];

S  = 25; % number of sim draws
S1 = 10; % ~ for initial estimate
N  = 15; % cross section
T  = 500;
B  = 500*T; % B/T number of B'strap draws   
ar       = .65; % ar coefficient for exogenous reg
rhoS     =  1; % 1 if spearman's rho is used
test =  1; % 1 if hypotheses tests are conducted
rng_type = 'threefry4x64_20';% any 'threefry4x64_20';%'twister';%'mlfg6331_64';% 
solver   = 'fmin';%'fmin';%
ini      = 'false'; %if true (false), then (no) grid search is used in addition to surrogate to obtain initials             

reg_type = "three"; F_type = "one"; % allowed combination reg_typ, F_type = ("one", "three"), ("one", "one"), ("three", "one")
eps_type = "norm"; % either "norm" or "st"
model = [reg_type, F_type, eps_type];

spmd
    rng(0,rng_type)
end

tic 
parfor (m=1:M,10)
    [theta_store(m,:), test_store(m,:), theta_store_opt(m,:), test_store_opt(m,:)]  = simulate_copula(theta0,q,LB,UB,T,S,S1,N,B, model,ar,test,rhoS,rng_type,ini,solver,m)                                          
end
minute = toc/60;
disp(strcat('Time past(min):' , num2str(minute)))
 
mean(theta_store,'omitnan')
mean(theta_store_opt,'omitnan')

median(theta_store,'omitnan')
median(theta_store_opt,'omitnan')

var(theta_store,'omitnan')
var(theta_store_opt,'omitnan')

mean(test_store,'omitnan')
mean(test_store_opt,'omitnan')

